==> node-etcd3-0.4.1.tgz: package/lib/etcd.js <==
"use strict";
var grpc = require("grpc");
var extend = require("lodash.assignin");
var deasyncPromise = require("deasync-promise");
var keepalivetoken_1 = require("./keepalivetoken");
var etcderror_1 = require("./etcderror");
var etcdProto = grpc.load(__dirname + "/../protos/rpc.proto");
var Etcd = (function () {
    function Etcd(servers, options) {
        if (servers === void 0) { servers = ["localhost:2379"]; }
        if (options === void 0) { options = {}; }
        this.credentials = grpc.credentials.createInsecure();
        this.keepAlives = [];
        this.options = extend(Etcd.defaults, options);
        this.servers = servers;
        if (this.servers.length > 1)
            console.warn("etcd3: currently only the first server address is used");
        this.clients = {
            KV: new etcdProto.etcdserverpb.KV(this.servers[0], this.credentials),
            Lease: new etcdProto.etcdserverpb.Lease(this.servers[0], this.credentials),
        };
    }
    Etcd.prototype.getBuffer = function (obj) {
        if (Buffer.isBuffer(obj))
            return obj;
        else if (obj == null)
            return new Buffer(0);
        else if (typeof obj === "string")
            return new Buffer(obj);
        else
            return new Buffer(JSON.stringify(obj));
    };
    Etcd.prototype.callClient = function (client, method, arg) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.clients[client][method](arg, function (err, response) {
                if (err) {
                    reject(new etcderror_1.EtcdError("etcd " + client + "." + method + " failed", err));
                    return;
                }
                resolve(response);
            });
        });
    };
    Etcd.prototype.callClientStream = function (client, method) {
        return this.clients[client][method]();
    };
    /**
     * closes the network connection and stops all keep alives started with this client.
     */
    Etcd.prototype.close = function () {
        for (var _i = 0, _a = this.keepAlives; _i < _a.length; _i++) {
            var token = _a[_i];
            token.cancel();
        }
        for (var _b = 0, _c = this.clients; _b < _c.length; _b++) {
            var client = _c[_b];
            var channel = grpc.getClientChannel(client);
            channel.close();
        }
    };
    /**
     * get the managed client lease. If it isn't created yet, it will be created. The Client lease will be kept alive automatic.
     * @return lease ID
     */
    Etcd.prototype.getClientLease = function () {
        var _this = this;
        if (this.clientLeasePromise) {
            return this.clientLeasePromise;
        }
        return this.clientLeasePromise = this.leaseGrant(this.options.appLeaseTtl).then(function (lease) {
            _this.clientLease = lease;
            _this.clientLeaseToken = _this.leaseKeepAlive(lease, _this.options.appLeaseKeepAlive);
            return lease;
        });
    };
    /**
     * blocking version of get()
     * @see {@link get}
     */
    Etcd.prototype.getSync = function (key, returnType) {
        return deasyncPromise(this.get(key, returnType));
    };
    Etcd.prototype.get = function (key, returnType) {
        if (returnType === void 0) { returnType = "value"; }
        return this.callClient("KV", "range", {
            key: new Buffer(key)
        }).then(function (res) {
            if (res.kvs.length) {
                switch (returnType) {
                    case "raw": return res.kvs[0];
                    case "buffer": return res.kvs[0].value;
                    case "value": return res.kvs[0].value.toString();
                    case "json": return JSON.parse(res.kvs[0].value.toString());
                }
            }
            else {
                // key not found
                return null;
            }
        });
    };
    /**
     * blocking version of range()
     * @see {@link range}
     */
    Etcd.prototype.rangeSync = function (fromKey, toKey, returnType) {
        return deasyncPromise(this.range(fromKey, toKey, returnType));
    };
    /**
     * get a range of key from etcd
     * @param fromKey - key to start from
     * @param toKey - key to end on
     * @return value of the key
     */
    Etcd.prototype.range = function (fromKey, toKey, returnType) {
        if (returnType === void 0) { returnType = "value"; }
        return this.callClient("KV", "range", {
            key: new Buffer(fromKey),
            range_end: new Buffer(toKey),
            limit: 500
        }).then(function (res) {
            var result = {};
            for (var _i = 0, _a = res.kvs; _i < _a.length; _i++) {
                var kv = _a[_i];
                var key = kv.key.toString();
                switch (returnType) {
                    case "raw":
                        result[key] = kv;
                        break;
                    case "buffer":
                        result[key] = kv.value;
                        break;
                    case "value":
                        result[key] = kv.value.toString();
                        break;
                    case "json":
                        result[key] = JSON.parse(kv.value.toString());
                        break;
                }
            }
            return result;
        });
    };
    /**
     * blocking version of set()
     * @see {@link set}
     */
    Etcd.prototype.setSync = function (key, value, lease) {
        return deasyncPromise(this.set(key, value, lease));
    };
    /**
     * set a key/value to etcd.
     * * a `Buffer` is saved as-is
     * * `null` is saved as an empty string
     * * `undefined` is saved as an empty string
     * * strings are saved as-is
     * * everthing else is tried to `JSON.stringify(value)`
     *
     * if you set "lease" to "client" a lease uniq to the instance of the Client will be used. This lease
     * will be created automatic and will be kept alive automatic.
     *
     * @param key - etcd key to set
     * @param value - content to set
     * @param lease - lease ID to use, or `client` to the client lease
     * @return lease ID used if any was used, else `null`
     */
    Etcd.prototype.set = function (key, value, lease) {
        var _this = this;
        var leaseId;
        var getLease = Promise.resolve(lease);
        if (typeof lease === "number") {
            getLease = this.leaseGrant(lease);
        }
        else if (lease === "client") {
            getLease = this.getClientLease();
        }
        return getLease.then(function (newLease) {
            leaseId = newLease;
            return _this.callClient("KV", "put", {
                key: new Buffer(key),
                value: _this.getBuffer(value),
                lease: leaseId
            });
        }).then(function (res) {
            return leaseId;
        });
    };
    /**
     * blocking version of delete()
     * @see {@link delete}
     */
    Etcd.prototype.deleteSync = function (key, keyTo) {
        return deasyncPromise(this.delete(key, keyTo));
    };
    /**
     * delete a key from etcd.
     * @param key - etcd key to delete
     * @return number of keys deleted
     */
    Etcd.prototype.delete = function (key, keyTo) {
        return this.callClient("KV", "deleteRange", {
            key: new Buffer(key),
            range_end: keyTo ? new Buffer(keyTo) : null
        }).then(function (res) {
            return res.deleted;
        });
    };
    /**
     * blocking version of leaseGrant()
     * @see {@link leaseGrant}
     */
    Etcd.prototype.leaseGrantSync = function (ttl) {
        return deasyncPromise(this.leaseGrant(ttl));
    };
    /**
     * request a new lease from etcd
     * @param ttl - requested ttl of the lease
     * @return
     */
    Etcd.prototype.leaseGrant = function (ttl) {
        return this.callClient("Lease", "leaseGrant", {
            TTL: ttl
        }).then(function (res) {
            return res.ID;
        });
    };
    /**
     * keep a lease alive
     * @param lease - lease to keep alive
     * @param interval - interval in which to send keep alives
     * @return id of interval used
     */
    Etcd.prototype.leaseKeepAlive = function (lease, interval) {
        if (interval === void 0) { interval = 1000; }
        var handler = this.callClientStream("Lease", "leaseKeepAlive");
        var token = new keepalivetoken_1.KeepAliveToken(handler, lease, interval);
        this.keepAlives.push(token);
        return token;
    };
    Etcd.defaults = {
        appLeaseTtl: 10,
        appLeaseKeepAlive: null,
    };
    return Etcd;
}());
exports.Etcd = Etcd;

==> node-etcd3-0.4.4.tgz: package/lib/etcd.js <==
"use strict";
var grpc = require("grpc");
var extend = require("lodash.assignin");
var deasyncPromise = require("deasync-promise");
var keepalivetoken_1 = require("./keepalivetoken");
var etcderror_1 = require("./etcderror");
var etcdProto = grpc.load(__dirname + "/../protos/rpc.proto");
/**
 * etcd client using the grpc protocol of etcd version 3 and later.
 */
var Etcd = (function () {
    /**
     * Creates a new etcd client.
     * @param servers
     * array of etcd hosts to use
     *
     * **Note**: currently only the first server is used, this will be fixed before version 1
     * @param options - additional options, see {@link defaults} for actual defaults
     */
    function Etcd(servers, options) {
        if (servers === void 0) { servers = ["localhost:2379"]; }
        if (options === void 0) { options = {}; }
        this.credentials = grpc.credentials.createInsecure();
        /** KeepAliveToken's created by this client */
        this.keepAlives = [];
        this.options = extend(Etcd.defaults, options);
        this.servers = servers;
        if (this.servers.length > 1)
            console.warn("etcd3: currently only the first server address is used");
        this.clients = {
            KV: new etcdProto.etcdserverpb.KV(this.servers[0], this.credentials),
            Lease: new etcdProto.etcdserverpb.Lease(this.servers[0], this.credentials),
        };
    }
    Etcd.prototype.getBuffer = function (obj) {
        if (Buffer.isBuffer(obj))
            return obj;
        else if (obj == null)
            return new Buffer(0);
        else if (typeof obj === "string")
            return new Buffer(obj);
        else
            return new Buffer(JSON.stringify(obj));
    };
    Etcd.prototype.callClient = function (client, method, arg) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.clients[client][method](arg, function (err, response) {
                if (err) {
                    reject(new etcderror_1.EtcdError("etcd " + client + "." + method + " failed", err));
                    return;
                }
                resolve(response);
            });
        });
    };
    Etcd.prototype.callClientStream = function (client, method) {
        return this.clients[client][method]();
    };
    /**
     * closes the network connection and stops all keep alives started with this client.
     */
    Etcd.prototype.close = function () {
        for (var _i = 0, _a = this.keepAlives; _i < _a.length; _i++) {
            var token = _a[_i];
            token.cancel();
        }
        for (var _b = 0, _c = this.clients; _b < _c.length; _b++) {
            var client = _c[_b];
            var channel = grpc.getClientChannel(client);
            channel.close();
        }
    };
    /**
     * get the managed client lease. If it isn't created yet, it will be created. The Client lease will be kept alive automatic.
     * @return lease ID
     */
    Etcd.prototype.getClientLease = function () {
        var _this = this;
        if (this.clientLeasePromise) {
            return this.clientLeasePromise;
        }
        return this.clientLeasePromise = this.leaseGrant(this.options.appLeaseTtl).then(function (lease) {
            _this.clientLease = lease;
            _this.clientLeaseToken = _this.leaseKeepAlive(lease, _this.options.appLeaseKeepAlive);
            return lease;
        });
    };
    /**
     * blocking version of get()
     * @see {@link get}
     */
    Etcd.prototype.getSync = function (key, returnType) {
        return deasyncPromise(this.get(key, returnType));
    };
    /**
     * get the value of a key from etcd
     * @param key - key to get
     * @param returnType
     * * `value` (default) - return the as a `string`
     * * `json` - parse as JSON and return the result
     * * `buffer` - return a NodeJS Buffer
     * * `raw` - return the value as it came from etcd
     * @return value of the key as defined by `returnType`
     */
    Etcd.prototype.get = function (key, returnType) {
        if (returnType === void 0) { returnType = "value"; }
        return this.callClient("KV", "range", {
            key: new Buffer(key)
        }).then(function (res) {
            if (res.kvs.length) {
                switch (returnType) {
                    default:
                    case "value": return res.kvs[0].value.toString();
                    case "raw": return res.kvs[0];
                    case "buffer": return res.kvs[0].value;
                    case "json": return JSON.parse(res.kvs[0].value.toString());
                }
            }
            else {
                // key not found
                return null;
            }
        });
    };
    /**
     * blocking version of range()
     * @see {@link range}
     */
    Etcd.prototype.rangeSync = function (fromKey, toKey, returnType) {
        return deasyncPromise(this.range(fromKey, toKey, returnType));
    };
    /**
     * get a range of key from etcd
     * @param fromKey - key to start from (inclusive)
     * @param toKey - key to end on (exclusive)
     * @param returnType
     * * `value` (default) - return the as a `string`
     * * `json` - parse as JSON and return the result
     * * `buffer` - return a NodeJS Buffer
     * * `raw` - return the value as it came from etcd
     * @return object with a key for each key returned, value of the key as defined by `returnType`
     */
    Etcd.prototype.range = function (fromKey, toKey, returnType) {
        if (returnType === void 0) { returnType = "value"; }
        return this.callClient("KV", "range", {
            key: new Buffer(fromKey),
            range_end: new Buffer(toKey),
            limit: 500
        }).then(function (res) {
            var result = {};
            for (var _i = 0, _a = res.kvs; _i < _a.length; _i++) {
                var kv = _a[_i];
                var key = kv.key.toString();
                switch (returnType) {
                    default:
                    case "value":
                        result[key] = kv.value.toString();
                        break;
                    case "raw":
                        result[key] = kv;
                        break;
                    case "buffer":
                        result[key] = kv.value;
                        break;
                    case "json":
                        result[key] = JSON.parse(kv.value.toString());
                        break;
                }
            }
            return result;
        });
    };
    /**
     * blocking version of set()
     * @see {@link set}
     */
    Etcd.prototype.setSync = function (key, value, lease) {
        return deasyncPromise(this.set(key, value, lease));
    };
    /**
     * set a key/value to etcd.
     * * a `Buffer` is saved as-is
     * * `null` is saved as an empty string
     * * `undefined` is saved as an empty string
     * * strings are saved as-is
     * * everything else is tried to `JSON.stringify(value)`
     *
     * if you set "lease" to "client" a lease uniq to the instance of the Client will be used. This lease
     * will be created automatic and will be kept alive automatic.
     *
     * @param key - etcd key to set
     * @param value - content to set
     * @param lease - lease ID to use, or `client` to the client lease
     * @return lease ID used if any was used, else `null`
     */
    Etcd.prototype.set = function (key, value, lease) {
        var _this = this;
        var leaseId;
        var getLease = Promise.resolve(lease);
        if (typeof lease === "number") {
            getLease = this.leaseGrant(lease);
        }
        else if (lease === "client") {
            getLease = this.getClientLease();
        }
        return getLease.then(function (newLease) {
            leaseId = newLease;
            return _this.callClient("KV", "put", {
                key: new Buffer(key),
                value: _this.getBuffer(value),
                lease: leaseId
            });
        }).then(function (res) {
            return leaseId;
        });
    };
    /**
     * blocking version of delete()
     * @see {@link delete}
     */
    Etcd.prototype.deleteSync = function (key, keyTo) {
        return deasyncPromise(this.delete(key, keyTo));
    };
    /**
     * delete a key from etcd.
     * @param key - key to start from (inclusive)
     * @param keyTo - key to end on (exclusive)
     * @return number of keys deleted
     */
    Etcd.prototype.delete = function (key, keyTo) {
        return this.callClient("KV", "deleteRange", {
            key: new Buffer(key),
            range_end: keyTo ? new Buffer(keyTo) : null
        }).then(function (res) {
            return res.deleted;
        });
    };
    /**
     * blocking version of leaseGrant()
     * @see {@link leaseGrant}
     */
    Etcd.prototype.leaseGrantSync = function (ttl) {
        return deasyncPromise(this.leaseGrant(ttl));
    };
    /**
     * request a new lease from etcd
     * @param ttl - requested TTL of the lease
     * @return
     */
    Etcd.prototype.leaseGrant = function (ttl) {
        return this.callClient("Lease", "leaseGrant", {
            TTL: ttl
        }).then(function (res) {
            return res.ID;
        });
    };
    /**
     * keep a lease alive
     * @param lease - lease to keep alive
     * @param interval - interval in which to send keep alives
     * @return `KeepAliveToken` that was created
     */
    Etcd.prototype.leaseKeepAlive = function (lease, interval) {
        if (interval === void 0) { interval = 1000; }
        var handler = this.callClientStream("Lease", "leaseKeepAlive");
        var token = new keepalivetoken_1.KeepAliveToken(handler, lease, interval);
        this.keepAlives.push(token);
        return token;
    };
    Etcd.defaults = {
        appLeaseTtl: 10,
        appLeaseKeepAlive: null,
    };
    return Etcd;
}());
exports.Etcd = Etcd;
